"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDiffFileList = exports.getRangesForDiff = exports.getIgnorePatterns = exports.getDiffForFile = void 0;
const child_process = __importStar(require("child_process"));
const path = __importStar(require("path"));
const Range_1 = require("./Range");
const sanitizeFilePath = (filePath) => JSON.stringify(path.resolve(filePath));
const diffCacheKey = (filePath, staged) => JSON.stringify([path.resolve(filePath), staged]);
const setCachedDiff = (filePath, staged, diff) => void diffCache.set(diffCacheKey(filePath, staged), diff);
const getCachedDiff = (filePath, staged) => diffCache.get(diffCacheKey(filePath, staged));
const diffCache = new Map();
const getDiffForFile = (filePath, staged = false) => {
    var _a;
    let diff = getCachedDiff(filePath, staged);
    if (diff === undefined) {
        const command = [
            "git",
            "diff",
            "--diff-filter=ACM",
            staged && "--staged",
            "--unified=0",
            JSON.stringify((_a = process.env.ESLINT_PLUGIN_DIFF_COMMIT) !== null && _a !== void 0 ? _a : "HEAD"),
            "--",
            sanitizeFilePath(filePath),
        ]
            .filter(Boolean)
            .join(" ");
        const result = child_process.execSync(command).toString();
        setCachedDiff(filePath, staged, result);
        diff = result;
    }
    return diff;
};
exports.getDiffForFile = getDiffForFile;
let diffFileListCache;
const getDiffFileList = (staged = false) => {
    var _a;
    if (diffFileListCache === undefined) {
        const command = [
            "git",
            "diff",
            "--diff-filter=ACM",
            "--name-only",
            staged && "--staged",
            JSON.stringify((_a = process.env.ESLINT_PLUGIN_DIFF_COMMIT) !== null && _a !== void 0 ? _a : "HEAD"),
        ]
            .filter(Boolean)
            .join(" ");
        diffFileListCache = child_process
            .execSync(command)
            .toString()
            .trim()
            .split("\n")
            .map((filePath) => path.resolve(filePath));
    }
    return diffFileListCache;
};
exports.getDiffFileList = getDiffFileList;
let gitFileListCache;
const getGitFileList = () => {
    if (gitFileListCache === undefined) {
        const command = ["git", "ls-files"].filter(Boolean).join(" ");
        gitFileListCache = child_process
            .execSync(command)
            .toString()
            .trim()
            .split("\n")
            .map((filePath) => path.resolve(filePath));
    }
    return gitFileListCache;
};
const getIgnorePatterns = (staged = false) => {
    const changedFiles = getDiffFileList(staged);
    const unchangedFiles = getGitFileList()
        .filter((x) => !changedFiles.includes(x))
        .map((x) => path.join("/", path.relative(process.cwd(), x)));
    return unchangedFiles;
};
exports.getIgnorePatterns = getIgnorePatterns;
const isHunkHeader = (input) => {
    const hunkHeaderRE = new RegExp(/^@@ .* @@/g);
    return input.match(hunkHeaderRE);
};
const getRangeForChangedLines = (line) => {
    var _a;
    const rangeRE = new RegExp(/^@@ .* \+(?<start>\d+)(?<linesCountDelimiter>,(?<linesCount>\d+))? @@/);
    const range = rangeRE.exec(line);
    if (range === null) {
        throw Error(`Couldn't match regex with line '${line}'`);
    }
    if (((_a = range.groups) === null || _a === void 0 ? void 0 : _a.start) === undefined) {
        /*
         * NOTE: Never happens, because RegExp requires start to be a
         * required number
         */
        throw Error("Couldn't match regex to find start");
    }
    const linesCount = range.groups.linesCountDelimiter && range.groups.linesCount
        ? parseInt(range.groups.linesCount)
        : 1;
    const hasAddedLines = linesCount !== 0;
    const start = parseInt(range.groups.start);
    const end = start + linesCount;
    return hasAddedLines ? new Range_1.Range(start, end) : null;
};
const removeNullRanges = (r) => r !== null;
const getRangesForDiff = (diff) => diff
    .split("\n")
    .filter(isHunkHeader)
    .map(getRangeForChangedLines)
    .filter(removeNullRanges);
exports.getRangesForDiff = getRangesForDiff;
